!*************************************************************************************************
! Climate statistics recorder
!*************************************************************************************************
SUBROUTINE climate_stat (tmp_air, tmp_soil, prec)

!_____________ Set variables
!Namespace
   USE data_structure
   USE time_counter
   USE vegi_status_current1
   USE vegi_status_record
   USE grid_status_current1
   USE grid_status_current2
   USE grid_status_record
   implicit none
   
!Augments
   real,intent(IN)::tmp_air   !2m air temperature (Celcius)
   real,intent(IN)::tmp_soil  !soil temperature (Celcius)
   real,intent(IN)::prec      !precipitation (mm day-1)
   
!Local variables
   real    tmp_air_month_ave      !monthly average of air temperature (Celcius)
   real    a1, a2
   integer count1, count2, i
   
!_____________ Main part
!Growth degree day
   if (doy==1) then
      gdd_20yr_RunningRecord(2:20) = gdd_20yr_RunningRecord(1:19)
      gdd_20yr_RunningRecord(1)    = gdd5
      gdd_20yr_ave = sum( gdd_20yr_RunningRecord(1:min(20, year)) ) / min(20, year)
      gdd0         = 0.0
      gdd5         = 0.0
   end if
   gdd0 = gdd0 + max(0.0, tmp_air    )
   gdd5 = gdd5 + max(0.0, tmp_air-5.0)
   
!!Temperature recorders for growth phases of grass PFTs
   if (doy==365) then
      
      a1 = 0.0 ; count1 = 0
      a2 = 0.0 ; count2 = 0
      do i=1, 365
         if (phenology_RunningRecord(C3g_no,i)) then
            a1     = a1 + tmp_air_RunningRecord(i)
            count1 = count1 + 1
         endif
         if (phenology_RunningRecord(C4g_no,i)) then
            a2     = a2 + tmp_air_RunningRecord(i)
            count2 = count2 + 1
         endif
      end do
      
      tmp_ave_C3growth_RunningRecord(2:20) = tmp_ave_C3growth_RunningRecord(1:19)
      tmp_ave_C4growth_RunningRecord(2:20) = tmp_ave_C4growth_RunningRecord(1:19)
      tmp_ave_C3growth_RunningRecord(1)    = a1 / max(1, count1)
      tmp_ave_C4growth_RunningRecord(1)    = a2 / max(1, count2)
      
   endif

!air temperature
   tmp_air_RunningRecord(2:365) = tmp_air_RunningRecord(1:364)
   tmp_air_RunningRecord(1)     = tmp_air
   
!soil temperature
   tmp_soil_RunningRecord(2:365) = tmp_soil_RunningRecord(1:364)
   tmp_soil_RunningRecord(1)     = tmp_soil
   tmp_soil_month_ave            = sum(tmp_soil_RunningRecord(1:30)) /30.0
   
!precipitation
   prec_RunningRecord(2:365) = prec_RunningRecord(1:364)
   prec_RunningRecord(1)     = prec
   
!coldest and hottest month of the year
   !update coldest and hottest month of the year
   if (Day_of_Month(doy) == Day_in_month(Month(doy))) then	!on the end of month
       tmp_air_month_ave = &
       sum( tmp_air_RunningRecord(1:Day_in_month(Month(doy))) ) / real( Day_in_month(Month(doy)) )
       
       if (Month(doy) == 1) then	!initialize on 31 January
           tmp_coldest_RunningRecord(2:20) = tmp_coldest_RunningRecord(1:19)
           tmp_coldest_RunningRecord(1)    = tmp_air_month_ave
           tmp_hottest_RunningRecord(2:20) = tmp_hottest_RunningRecord(1:19)
           tmp_hottest_RunningRecord(1)    = tmp_air_month_ave
       else
           tmp_coldest_RunningRecord(1) = min(tmp_coldest_RunningRecord(1), tmp_air_month_ave)
           tmp_hottest_RunningRecord(1) = max(tmp_hottest_RunningRecord(1), tmp_air_month_ave)
       end if
   end if
   
   !update 20 year running mean
   if (doy == 365) then	!on the end of year
      tmp_coldest_20yr_ave = sum( tmp_coldest_RunningRecord (1:min(20,year)) ) /min(20, year)
      tmp_hottest_20yr_ave = sum( tmp_hottest_RunningRecord (1:min(20,year)) ) /min(20, year)
   end if
   
   !running record of par_grass
   par_grass_RunningRecord(2:365) = par_grass_RunningRecord(1:364)
   par_grass_RunningRecord(1)     = par_grass
   
END SUBROUTINE climate_stat



!**************************************************************************************************
! Search existing Plant Functiona Type in the grid
!**************************************************************************************************
SUBROUTINE pft_present ()

!_____________ Set variables
!Namespace
   USE data_structure
   USE vegi_status_current1
   implicit none
   
!Local variables
   integer	p, no	!loop counter
   
!_____________ Main part
!Reset varialbes
   pft_exist(:)      = .false.
   pft_exist(C3g_no) = .true. 
   pft_exist(C4g_no) = .true. 
   
!Servey for existance of woody PFTs
   do no=1, Max_no
   if (tree_exist(no)) then
      pft_exist(pft(no)) = .true. 
   end if
   end do
   
END SUBROUTINE pft_present



!*************************************************************************************************
! Biome detemination routine (Must be called @ doy=365)
! -- original criteria was taken from BIOME3 of A. Haxeltine & I.C. Prentice (1996) --
!*************************************************************************************************
SUBROUTINE biome_determine (W_fi, W_wilt)

!_____________ Set variables
!Namespace
   USE data_structure
   USE vegi_status_current1
   USE vegi_status_record
   USE grid_status_current1
   USE grid_status_record
   implicit none
   
!Auguments
   real,intent(IN) ::W_fi     !filed capacity   (m3/m3, 0.0 -> 1.0)
   real,intent(IN) ::W_wilt   !wilting point    (m3/m3, 0.0 -> 1.0)
   
!Local variables
   real,dimension(12)::sm     !soil moisture index for each month
   real              ::sm_min !minimum value of sm
   real              ::sm_ave !average value of sm
   
   integer dominant1, dominant2 !
   real    lai_max              !annual maximum of LAI (wood + grass)
   integer p,i,j,k              !for general usage
   real    x, y                 !for general usage
   
!_____________ Preparare indices
!Maximum LAI
   lai_max = lai_max_wood + lai_max_grass
   
!First dominant PFT
   dominant1 = 1   !default
   x         = 0.0 
   do p = 1, PFT_no
      if (npp_annual(p) > x ) then
         x         = npp_annual(p)
         dominant1 = p
      endif
   enddo
   
!Secound dominant PFT
   dominant2 = 1   !default
   x         = 0.0 
   do p = 1, PFT_no
      if ( p .ne. dominant1 ) then
      if (npp_annual(p) > x ) then
         x         = npp_annual(p)
         dominant2 = p
      endif
      endif
   enddo
   
!sm(:): Soil moisture index of upper-soil-layer for each month
   do i = 1, 12                   !i: month
      k = sum(Day_in_month(1:i))  !k: ending doy of month i
      j = k - Day_in_month(i) + 1 !j: starting doy of month i
      x = sum(pool_w1_RunningRecord(j:k)) / real(k-j+1)
      sm(i) = (x/Depth-W_wilt) / (W_fi-W_wilt)
      sm(i) = min( 1.0, max(0.0, sm(i)) )
   enddo
   sm_min = min( sm(1),sm(2),sm(3),sm(4),sm(5),sm(6),sm(7),sm(8),sm(9),sm(10),sm(11),sm(12) )
   sm_ave = sum( sm(1:12) ) / 12.0
   
!_____________ Determine biome type

!----------- Group 1 -----------
   !1: Polar desert
   if (gdd0 < 150) then
      biome = 1; return
   endif
   
!----------- Group 2 -----------
   !2: Arctic/Alpine-tundra
   if (gdd5 < 370) then
      biome = 2; return
   endif
   
!----------- Group 3 -----------
SELECT CASE (dominant1)
   !3: tropical rain forest
   !4: tropical seasonal forest
   CASE (1:4)
      if (lai_max>=2.5) then
         if (sm_min >= 0.20) then
            biome = 3; return
         else
            biome = 4; return
         endif
      endif
   
   !5: tropical deciduous forest
   CASE (5)
      if (lai_max>=2.5) then
         biome = 5; return
      endif
   
   !6: temperate conifer forest
   CASE (6)
      if (lai_max>=1.5) then
         biome = 6; return
      endif
   
   !7: temperate broad-leaved evergreen forest
   CASE (7)
      if (lai_max>=3.0) then
         biome = 7; return
      endif
   
   !8: temperate deciduous forest
   CASE (8)
      if (lai_max>=2.5 ) then
         biome = 8; return
      endif
   
   !9: boreal evergreen forest
   CASE (9)
         biome = 9; return
   
   !10: boreal deciduous forest
   CASE (10:11)
         biome = 10; return
   
   !11: short grass land
   !12: tall grass land
   CASE (12)
      if (lai_max_grass>1.5 .and. sm_ave<0.65) then
         if ( lai_max_grass<3.0 ) then
            biome = 11; return
         else
            biome = 12; return
         endif
      endif
   
   CASE (13)
      x = npp_annual(C3g_no) + npp_annual(C4g_no) !annual grass NPP
      y = max( 1.0 , sum(npp_annual(:))-x )     !annual woody NPP
      x = x / y                                 !annual grass-NPP / woody-NPP
      
      if (lai_max_grass>0.4 .and. x>1.8) then
         if ( lai_max_grass<3.0 ) then
            biome = 11; return
         else
            biome = 12; return
         endif
      endif
   
END SELECT
   
!----------- Group 4 -----------
   !13: moist savannas
   if ( (dominant1==C3g_no .or. dominant1==C4g_no) .and. lai_max>=1.5 )then
      biome = 13; return
   endif
   
   !14: dry savannas
   if ( (dominant1==C3g_no .or. dominant1==C4g_no) .and. lai_max>=0.6 )then
      biome = 14; return
   endif
   
   !15: xeric woodland / scrub
   if (dominant1==1 .or. dominant1==2 .or. dominant1==3 .or. dominant1==4 .or. dominant1==5 .or. dominant1==7) then
   if (lai_max>=1.0 ) then
      biome = 15; return
   endif
   endif
   
   if (dominant1==6 .or. dominant1==7 .or. dominant1==9 .or. dominant1==10 .or. dominant1==11) then
   if (lai_max>=1.5 ) then
      biome = 15; return
   endif
   endif
   
!----------- Group 5 -----------
   !16: Arid shrubland / steppe
   !17: Desert
   if ( lai_max>=0.2 ) then
      biome = 16
   else
      biome = 17
   endif
   
END SUBROUTINE biome_determine



!*************************************************************************************************
! output spinup files
!*************************************************************************************************
SUBROUTINE spinup_out (Fn)
!_____________ Set variables
!Namespace
   USE data_structure
   USE time_counter
   USE vegi_status_current1
   USE vegi_status_record
   USE grid_status_current1
   USE grid_status_record
   implicit none
   
!Arguments
   integer,intent(IN)::Fn !File I/O number
   
!Local variables
   integer           n ,i, j, k
   character(len=3)  string
   
!_____________ Main part
write (Fn,*) Spinup_year+Simulation_year
write (Fn,*) phenology(:)
write (Fn,*) pft_exist(:)
write (Fn,*) tree_exist(:)

write (Fn,'(3(i5,1x),i7,1x,1(i5,1x) )'  ) type_grass, C3g_no, C4g_no, fire_clo, biome
write (Fn,'(31(f10.2,1x))') pool_w(:), pool_snow
write (Fn,'(3(f17.1,1x))') pool_litter, pool_som_int, pool_som_slow
write (Fn,'(f8.5,1x,f9.1,1x,f9.1)') canopy_cond, gdd0, gdd5
write (Fn,'(2(f15.1,1x))') c_emission, c_uptake
write (Fn,'(1(f10.5,1x))') tmp_soil_month_ave
write (Fn,'(3(f10.5,1x))') tmp_coldest_20yr_ave, tmp_hottest_20yr_ave, gdd_20yr_ave
write (Fn,'(365(f10.5,1x))') hyd_ro_RunningRecord(:)
write (Fn,'(365(f10.5,1x))') hyd_ic_RunningRecord(:)
write (Fn,'(365(f10.5,1x))') hyd_ev_RunningRecord(:)
write (Fn,'(365(f10.5,1x))') hyd_tr_RunningRecord(:)
write (Fn,'(4(f10.5,1x))') lai_max_wood, lai_max_grass, lai_mean_wood, lai_mean_grass

i = int(  PFT_no / 100      ) ; string(1:1)=char(i+48)
j = int( (PFT_no-i*100)/10  ) ; string(2:2)=char(j+48)
k = int(  PFT_no-i*100-j*10 ) ; string(3:3)=char(k+48)

write (Fn,'('//string//'(i7,1x))'    ) pheno_clo(:)
write (Fn,'('//string//'(i7,1x))'    ) pheno_cls(:)
write (Fn,'('//string//'(f6.4,1x))'  ) stat_water(:)
write (Fn,'('//string//'(f10.1,1x))' ) npp_10d_ave(:)

Do i=1, 365
   write (Fn,'('//string//'(f10.1,1x))') npp_RunningRecord(:,i)
End do

Do i=1, 365
   write (Fn,'('//string//'(f5.3,1x))') stat_water_RunningRecord(:,i)
End do

Do i=1, 365
   write (Fn, *) phenology_RunningRecord(:,i)
End do

write (Fn,'( 20(f7.2,1x))') gdd_20yr_RunningRecord(:)

write (Fn,'( 20(f6.2,1x))') tmp_coldest_RunningRecord(:)
write (Fn,'( 20(f6.2,1x))') tmp_hottest_RunningRecord(:)
write (Fn,'( 20(f6.2,1x))') tmp_ave_C3growth_RunningRecord(:)
write (Fn,'( 20(f6.2,1x))') tmp_ave_C4growth_RunningRecord(:)
write (Fn,'(365(f6.2,1x))') tmp_soil_RunningRecord(:)
write (Fn,'(365(f6.2,1x))') tmp_air_RunningRecord(:)

write (Fn,'(365(f6.2,1x))') prec_RunningRecord(:)
write (Fn,'(365(f6.1,1x))') pool_w1_RunningRecord(:)
write (Fn,'( 30(f6.1,1x))') par_RunningRecord(:)
write (Fn,'(365(f6.1,1x))') par_grass_RunningRecord(:)

write (Fn,'(12(f12.3,1x))') gmass_leaf(:), gmass_root(:), gmass_available(:), gmass_stock(:), lai_grass(:), lai_opt_grass(:)
write (Fn,'(40(f5.2,1x))') lai_opt_grass_RunningRecord(1,:), lai_opt_grass_RunningRecord(2,:)

Do n=1, Max_no
if (tree_exist(n)) then
   write (Fn,'(5(i5,1x) )'  ) pft(n), age(n), height(n), bole(n), height_limit(n)
   write (Fn,'(4(f10.5,1x))') dbh_heartwood(n), dbh_sapwood(n), crown_diameter(n), crown_area(n)
   write (Fn,'(6(f12.2,1x))') la(n), mass_leaf(n), mass_trunk(n), mass_root(n), mass_stock(n), mass_available(n)
   write (Fn,'(5(f10.5,1x))') bole_x(n), bole_y(n), crown_x(n), crown_y(n), radius_limit(n)
   write (Fn,'(3(f10.5,1x))') stat_leaf(n), stat_leaf_crowntop(n), mort_regu2(n)
   write (Fn,'(2(f12.3,1x), f10.8)') gpp_month_ind(n), mort_regu1(n), mort_regu4(n)
   write (Fn,'(10(f10.5,1x))') stat_leaf_crownbottom(n,:)
endif
End do

Do n=1, PFT_no
if (pft_exist(n)) then
   write (Fn,'(2(f15.5,1x))') gpp(n), npp(n)
   write (Fn,'(4(f15.5,1x))') gpp_month(n), npp_month(n), gpp_annual(n), npp_annual(n)
endif
Enddo

END SUBROUTINE spinup_out



!*************************************************************************************************
! output spinup files
!*************************************************************************************************
SUBROUTINE spinup_in (Fn)

!_____________ Set variables
!Namespace
   USE data_structure
   USE time_counter
   USE vegi_status_current1
   USE vegi_status_record
   USE grid_status_current1
   USE grid_status_record
   implicit none
   
!Arguments
   integer,intent(IN)::Fn !File I/O number
   
!Local variables
   integer n ,i
   
!_____________ Main part
!Read data
read (Fn,*) Spinup_year
read (Fn,*) phenology (:)
read (Fn,*) pft_exist (:)
read (Fn,*) tree_exist(:)

read (Fn,*) type_grass, C3g_no, C4g_no, fire_clo, biome
read (Fn,*) pool_w(:), pool_snow
read (Fn,*) pool_litter, pool_som_int, pool_som_slow
read (Fn,*) canopy_cond, gdd0, gdd5
read (Fn,*) c_emission, c_uptake
read (Fn,*) tmp_soil_month_ave
read (Fn,*) tmp_coldest_20yr_ave, tmp_hottest_20yr_ave, gdd_20yr_ave
read (Fn,*) hyd_ro_RunningRecord(:)
read (Fn,*) hyd_ic_RunningRecord(:)
read (Fn,*) hyd_ev_RunningRecord(:)
read (Fn,*) hyd_tr_RunningRecord(:)
read (Fn,*) lai_max_wood, lai_max_grass, lai_mean_wood, lai_mean_grass

read (Fn,*) pheno_clo(:)
read (Fn,*) pheno_cls(:)
read (Fn,*) stat_water(:)
read (Fn,*) npp_10d_ave(:)

Do i=1, 365
   read (Fn,*) npp_RunningRecord(:,i)
End do

Do i=1, 365
   read (Fn,*) stat_water_RunningRecord(:,i)
End do

Do i=1, 365
   read (Fn,*) phenology_RunningRecord(:,i)
End do

read (Fn,*) gdd_20yr_RunningRecord(:)

read (Fn,*) tmp_coldest_RunningRecord(:)
read (Fn,*) tmp_hottest_RunningRecord(:)
read (Fn,*) tmp_ave_C3growth_RunningRecord(:)
read (Fn,*) tmp_ave_C4growth_RunningRecord(:)
read (Fn,*) tmp_soil_RunningRecord(:)
read (Fn,*) tmp_air_RunningRecord(:)

read (Fn,*) prec_RunningRecord(:)
read (Fn,*) pool_w1_RunningRecord(:)
read (Fn,*) par_RunningRecord(:)
read (Fn,*) par_grass_RunningRecord(:)

read (Fn,*) gmass_leaf(:), gmass_root(:), gmass_available(:), gmass_stock(:), lai_grass(:), lai_opt_grass(:)
read (Fn,*) lai_opt_grass_RunningRecord(1,:), lai_opt_grass_RunningRecord(2,:)

Do n=1, Max_no
if (tree_exist(n)) then
   read (Fn,*) pft(n), age(n), height(n), bole(n), height_limit(n)
   read (Fn,*) dbh_heartwood(n), dbh_sapwood(n), crown_diameter(n), crown_area(n)
   read (Fn,*) la(n), mass_leaf(n), mass_trunk(n), mass_root(n), mass_stock(n), mass_available(n)
   read (Fn,*) bole_x(n), bole_y(n), crown_x(n), crown_y(n), radius_limit(n)
   read (Fn,*) stat_leaf(n), stat_leaf_crowntop(n), mort_regu2(n)
   read (Fn,*) gpp_month_ind(n), mort_regu1(n), mort_regu4(n)
   read (Fn,*) stat_leaf_crownbottom(n,:)
endif
End do

Do n=1, PFT_no
if (pft_exist(n)) then
   read (Fn,*) gpp(n), npp(n)
   read (Fn,*) gpp_month(n), npp_month(n), gpp_annual(n), npp_annual(n)
endif
Enddo

END SUBROUTINE spinup_in
